/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package org.processmining.xeslite.external;

import org.deckfour.xes.model.XAttributeMap;

/**
 * A store that stores {@link ExternalIdentifyable} objects.
 * 
 * @author F. Mannhardt
 * 
 */
public interface ExternalStore {

	/**
	 * @param attributable
	 *            that the attributes should be returned for
	 * @return the attributes (never returns NULL!)
	 */
	XAttributeMap getAttributes(ExternalAttributable attributable);

	/**
	 * @param attributable
	 *            that the attributes should be attached to
	 * @param attributes
	 *            the attributes that replace all existing attributes
	 */
	void setAttributes(ExternalAttributable attributable, XAttributeMap attributes);

	/**
	 * @param attributable
	 *            whose attributes should be removed
	 * @return the attributes that have been deleted
	 */
	XAttributeMap removeAttributes(ExternalAttributable attributable);

	/**
	 * @param attributable
	 * @return
	 */
	boolean hasAttributes(ExternalAttributable attributable);

	/**
	 * @return a {@link StringPool} that is used to map key {@link String}s to
	 *         {@link Integer}s
	 */
	StringPool getAttributeKeyPool();

	/**
	 * @return a {@link StringPool} that is used to map general literals
	 *         {@link String}s to {@link Integer}s
	 */
	StringPool getLiteralPool();

	/**
	 * @return a {@link IdFactory} that can be used to obtain new 'unique'
	 *         identifiers for objects stored in the {@link ExternalStorage}
	 */
	IdFactory getIdFactory();

	/**
	 * Commits all pending changes to the underlying storage
	 */
	void commit();

	//void saveLog(XLogExternalImpl log);

}